@@ -30,7 +30,9 @@
 
 #include <QtTest/QtTest>
 #include <QMetaType>
+#if QT_CONFIG(process)
 #include <QProcess>
+#endif
 #include <QFileInfo>
 #include <QTcpServer>
 #include <QTcpSocket>
@@ -52,7 +54,7 @@
 #define SET_NODE_NAME(obj) (obj).setName(QLatin1String(#obj))
 
 //DUMMY impl for variant comparison
-bool operator<(const QVector<int> &lhs, const QVector<int> &rhs)
+bool operator<(const QList<int> &lhs, const QList<int> &rhs)
 {
     return lhs.size() < rhs.size();
 }
@@ -215,11 +217,15 @@ private slots:
         QTest::newRow("qnx") << QUrl(QLatin1String("qnx:replica")) << QUrl(QLatin1String("qnx:registry"));
 #endif
         QTest::newRow("local") << QUrl(QLatin1String("local:replicaLocalIntegration")) << QUrl(QLatin1String("local:registryLocalIntegration"));
+#ifdef Q_OS_LINUX
+        QTest::newRow("localabstract") << QUrl(QLatin1String("localabstract:replicaAbstractIntegration")) << QUrl(QLatin1String("localabstract:registryAbstractIntegration"));
+#endif
         QTest::newRow("external") << QUrl() << QUrl();
     }
 
     void initTestCase()
     {
+        QVERIFY(TestUtils::init("integration"));
         QLoggingCategory::setFilterRules("qt.remoteobjects.warning=false");
 
         // use different paths in QRemoteObjectSettingsStore
@@ -338,7 +344,7 @@ private slots:
         const QScopedPointer<TestClassReplica> tc_rep(client->acquire<TestClassReplica>());
         tc_rep->waitForSource();
         QCOMPARE(tc.testEnum(), tc_rep->testEnum());
-        QCOMPARE((qint32)tc.classEnum(), (qint32)TestClassSimpleSource::One);
+        QCOMPARE(qint32(tc.classEnum()), qint32(TestClassSimpleSource::One));
 
         // set property on the replica (test property change packet)
         {
@@ -347,7 +353,7 @@ private slots:
             tc_rep->pushClassEnum(TestClassReplica::Two);
             QVERIFY(spy.count() || spy.wait());
 
-            QCOMPARE((qint32)tc.classEnum(), (qint32)tc_rep->classEnum());
+            QCOMPARE(qint32(tc.classEnum()), qint32(tc_rep->classEnum()));
         }
 
         // set property on the source (test property change packet)
@@ -356,7 +362,7 @@ private slots:
             tc.setClassEnum(TestClassSimpleSource::One);
             QVERIFY(spy.wait());
 
-            QCOMPARE((qint32)tc.classEnum(), (qint32)tc_rep->classEnum());
+            QCOMPARE(qint32(tc.classEnum()), qint32(tc_rep->classEnum()));
         }
 
         QScopedPointer<QRemoteObjectDynamicReplica> tc_repDynamic(client->acquireDynamic(QStringLiteral("TestClass")));
@@ -371,7 +377,7 @@ private slots:
 
         QMetaProperty property = metaObject->property(propertyIndex);
         QVERIFY(property.isValid());
-        QCOMPARE(property.typeName(), "ClassEnum");
+        QCOMPARE(property.typeName(), "TestClassReplica::ClassEnum");
 
         // read enum on the dynamic replica
         {
@@ -392,7 +398,7 @@ private slots:
 
         property = metaObject->property(propertyIndex);
         QVERIFY(property.isValid());
-        QCOMPARE(property.typeName(), "ClassEnum");
+        QCOMPARE(property.typeName(), "TestClassReplica::ClassEnum");
 
         // read enum on the dynamic replica
         {
@@ -479,6 +485,90 @@ private slots:
         QCOMPARE(instances, QStringList({"Engine2"}));
     }
 
+    void registrySourceLocationBindings()
+    {
+        QFETCH_GLOBAL(QUrl, registryUrl);
+        QFETCH_GLOBAL(QUrl, hostUrl);
+        if (registryUrl.isEmpty())
+            QSKIP("Skipping registry tests for external QIODevice types.");
+
+        setupRegistry();
+        setupHost(true);
+        setupClient(true);
+
+        QVERIFY(host->registry()->sourceLocations().empty());
+        QVERIFY(client->registry()->sourceLocations().empty());
+
+        QVERIFY(host->registry()->bindableSourceLocations().isReadOnly());
+        QVERIFY(client->registry()->bindableSourceLocations().isReadOnly());
+
+        Engine e1;
+        const auto engine1 = QStringLiteral("Engine1");
+        Engine e2;
+        const auto engine2 = QStringLiteral("Engine2");
+
+        QRemoteObjectSourceLocations expectedSourceLocations;
+        expectedSourceLocations[engine1] = { QStringLiteral("Engine"), hostUrl };
+
+        int hostSrcLocationsChanged = 0;
+        auto hostHandler = host->registry()->bindableSourceLocations().onValueChanged([&] {
+            QCOMPARE(host->registry()->sourceLocations(), expectedSourceLocations);
+            ++hostSrcLocationsChanged;
+        });
+
+        int clientSrcLocationsChanged = 0;
+        auto clientHandler = client->registry()->bindableSourceLocations().onValueChanged([&] {
+            QCOMPARE(client->registry()->sourceLocations(), expectedSourceLocations);
+            ++clientSrcLocationsChanged;
+        });
+
+        QProperty<QRemoteObjectSourceLocations> hostObserver;
+        hostObserver.setBinding([&] { return host->registry()->sourceLocations(); });
+
+        QProperty<QRemoteObjectSourceLocations> clientObserver;
+        clientObserver.setBinding([&] { return client->registry()->sourceLocations(); });
+
+        QSignalSpy hostSpy(host->registry(), &QRemoteObjectRegistry::remoteObjectAdded);
+        QSignalSpy clientSpy(client->registry(), &QRemoteObjectRegistry::remoteObjectAdded);
+
+        host->enableRemoting(&e1, engine1);
+        QTRY_COMPARE(hostSpy.count(), 1);
+        QTRY_COMPARE(clientSpy.count(), 1);
+        QCOMPARE(hostObserver.value(), host->registry()->sourceLocations());
+        QCOMPARE(clientObserver.value(), client->registry()->sourceLocations());
+        QCOMPARE(hostObserver.value(), clientObserver.value());
+        QCOMPARE(hostObserver.value(), expectedSourceLocations);
+        QCOMPARE(hostSrcLocationsChanged, 1);
+        QCOMPARE(clientSrcLocationsChanged, 1);
+
+        expectedSourceLocations[engine2] = { QStringLiteral("Engine"), hostUrl };
+        host->enableRemoting(&e2, engine2);
+        QTRY_COMPARE(hostSpy.count(), 2);
+        QTRY_COMPARE(clientSpy.count(), 2);
+        QCOMPARE(hostObserver.value(), host->registry()->sourceLocations());
+        QCOMPARE(clientObserver.value(), client->registry()->sourceLocations());
+        QCOMPARE(hostObserver.value(), clientObserver.value());
+        QCOMPARE(hostObserver.value(), expectedSourceLocations);
+        QCOMPARE(hostSrcLocationsChanged, 2);
+        QCOMPARE(clientSrcLocationsChanged, 2);
+
+        // Test source removal
+        host->disableRemoting(&e1);
+        expectedSourceLocations.remove(engine1);
+        QSignalSpy srcRemovedHostSpy(host->registry(), &QRemoteObjectRegistry::remoteObjectRemoved);
+        QSignalSpy srcRemovedClientSpy(client->registry(),
+                                       &QRemoteObjectRegistry::remoteObjectRemoved);
+
+        QTRY_COMPARE(srcRemovedHostSpy.count(), 1);
+        QTRY_COMPARE(srcRemovedClientSpy.count(), 1);
+        QCOMPARE(hostObserver.value(), host->registry()->sourceLocations());
+        QCOMPARE(clientObserver.value(), client->registry()->sourceLocations());
+        QCOMPARE(hostObserver.value(), clientObserver.value());
+        QCOMPARE(hostObserver.value(), expectedSourceLocations);
+        QCOMPARE(hostSrcLocationsChanged, 3);
+        QCOMPARE(clientSrcLocationsChanged, 3);
+    }
+
     void registryAddedTest()
     {
         QFETCH_GLOBAL(QUrl, registryUrl);
@@ -557,7 +647,7 @@ private slots:
         source1.setData1(5);
         source1.setData2(5.0);
         source1.setData3(QStringLiteral("tcp"));
-        source1.setData4(QVector<int>() << 1 << 2 << 3 << 4 << 5);
+        source1.setData4(QList<int> { 1, 2, 3, 4, 5 });
         registry->enableRemoting(&source1);
 
         setupHost(true);
@@ -565,7 +655,7 @@ private slots:
         source2.setData1(5);
         source2.setData2(5.0);
         source2.setData3(QStringLiteral("local"));
-        source2.setData4(QVector<int>() << 1 << 2 << 3 << 4 << 5);
+        source2.setData4(QList<int> { 1, 2, 3, 4, 5 });
         host->enableRemoting(&source2);
         QVERIFY(host->waitForRegistry(1000));
 
@@ -581,15 +671,16 @@ private slots:
         QTRY_VERIFY(localCentre->isInitialized());
         QTRY_VERIFY(tcpCentre->isInitialized());
 
+        const QList<int> expected = { 1, 2, 3, 4, 5 };
         QCOMPARE(tcpCentre->data1(), 5 );
         QCOMPARE(tcpCentre->data2(), 5.0);
         QCOMPARE(tcpCentre->data3(), QStringLiteral("tcp"));
-        QCOMPARE(tcpCentre->data4(), QVector<int>() << 1 << 2 << 3 << 4 << 5);
+        QCOMPARE(tcpCentre->data4(), expected);
 
         QCOMPARE(localCentre->data1(), 5);
         QCOMPARE(localCentre->data2(), 5.0);
         QCOMPARE(localCentre->data3(), QStringLiteral("local"));
-        QCOMPARE(localCentre->data4(), QVector<int>() << 1 << 2 << 3 << 4 << 5);
+        QCOMPARE(localCentre->data4(), expected);
     }
 
     void invalidUrlsTest()
@@ -623,7 +714,8 @@ private slots:
     {
         QFETCH_GLOBAL(QUrl, registryUrl);
         if (registryUrl.isEmpty())
-            QSKIP("Skipping registry tests for external QIODevice types.");        setupHost(true);
+            QSKIP("Skipping registry tests for external QIODevice types.");
+        setupHost(true);
         const bool res = host->waitForRegistry(3000);
         QVERIFY(!res);
         QCOMPARE(host->registry()->isInitialized(), false);
@@ -826,7 +918,7 @@ private slots:
         QCOMPARE(call.error(), QRemoteObjectPendingCall::InvalidMessage);
         QVERIFY(call.waitForFinished());
         QVERIFY(call.isFinished());
-        QCOMPARE(call.returnValue().type(), QVariant::Bool);
+        QCOMPARE(call.returnValue().metaType(), QMetaType::fromType<bool>());
         QCOMPARE(call.returnValue().toBool(), true);
         started = property.read(engine_r.data()).value<bool>();
         QCOMPARE(started, true);
@@ -872,7 +964,7 @@ private slots:
         QVERIFY(getCall.waitForFinished());
         QVERIFY(getCall.isFinished());
         QCOMPARE(getCall.error(), QRemoteObjectPendingCall::NoError);
-        QCOMPARE(getCall.returnValue().type(), QVariant::String);
+        QCOMPARE(getCall.returnValue().metaType(), QMetaType::fromType<QString>());
         QCOMPARE(getCall.returnValue().toString(), s);
     }
 
@@ -1323,15 +1415,13 @@ private slots:
         QVERIFY(invalidRegistry.lastError() == QRemoteObjectNode::RegistryNotAcquired);
     }
 
-#if defined(Q_OS_LINUX) || defined(Q_OS_DARWIN)
+#if QT_CONFIG(process) && (defined(Q_OS_LINUX) || defined(Q_OS_DARWIN))
     void localServerConnectionTest()
     {
         QFETCH_GLOBAL(QUrl, hostUrl);
         if (hostUrl.scheme() != QRemoteObjectStringLiterals::local())
             QSKIP("Skipping 'local' specific backend for non-local test.");
-        const auto progName = TestUtils::findExecutable("localsockettestserver", {
-            QCoreApplication::applicationDirPath() + "/../localsockettestserver"
-        });
+        const auto progName = TestUtils::findExecutable("localsockettestserver", "/localsockettestserver");
 
         //create a fake socket as killing doesn't produce a necessarily unusable socket
         QFile fake(QDir::temp().absoluteFilePath(QStringLiteral("crashMe")));
@@ -1363,9 +1453,7 @@ private slots:
         QFETCH_GLOBAL(QUrl, hostUrl);
         if (hostUrl.scheme() != QRemoteObjectStringLiterals::local())
             QSKIP("Skipping 'local' specific backend for non-local test.");
-        const auto progName = TestUtils::findExecutable("localsockettestserver", {
-            QCoreApplication::applicationDirPath() + "/../localsockettestserver"
-        });
+        const auto progName = TestUtils::findExecutable("localsockettestserver", "/localsockettestserver");
 
         QProcess testServer;
         testServer.start(progName, QStringList());
@@ -1423,6 +1511,23 @@ private slots:
         QVERIFY(res == false);
     }
 
+    void startClientWithoutHost()
+    {
+        setupClient();
+        QScopedPointer<EngineReplica> replica(client->acquire<EngineReplica>());
+        client->setHeartbeatInterval(10);
+        // Wait, to make sure there's no crash (QTBUG-94513)
+        QTest::qWait(200);
+
+        // Make sure creating the host afterwards works
+        setupHost();
+        Engine e;
+        e.setRpm(42);
+        host->enableRemoting(&e);
+
+        QVERIFY(replica->waitForSource());
+        QCOMPARE(replica->rpm(), e.rpm());
+    }
 };
 
 QTEST_MAIN(tst_Integration)
